/************************************************************************/
/*																		*/
/*			Access Dallas 1-Wire Devices								*/
/*                                                                      */
/*              Author: Peter Dannegger                                 */
/*                      danni@specs.de                                  */
/*                                                                      */
/************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "1wire.h"

/*	wait_x_075u:
	Bei Parameter n=0 dauert die Schleife 7,5 us
	Bei Parameter n>0 dauert die Schleife (n*2,75+5,5) us
*/

bit w1_reset(void)
{
  bit err;

  W1_OUT &= ~(1<<W1_PIN);
  W1_DDR |= 1<<W1_PIN;
  wait_x_075u( 216 );			// 599,5 us
  cli();
  W1_DDR &= ~(1<<W1_PIN);
  wait_x_075u( 23 );			// 68,75 us
  err = W1_IN & (1<<W1_PIN);			// no presence detect
  sei();
  wait_x_075u( 136 );		// 379,5 us
  if( (W1_IN & (1<<W1_PIN)) == 0 )		// short circuit
    err = 1;
  return err;
}


uchar w1_bit_io( bit b, bit pull )
{
  cli();
  W1_DDR |= (1<<W1_PIN);
  wait_x_1u();		// 1 us
  wait_x_1u();		// 1 us
  if( b )
    W1_DDR &= ~(1<<W1_PIN);
  wait_x_075u( 0 );		// 7,5 us
  if( (W1_IN & (1<<W1_PIN)) == 0 )
    b = 0;
  wait_x_075u( 17 );	// 52,25 us -> Gesamtzeit 61,75
  if( pull ) {
	W1_OUT |= (1<<W1_PIN);
	W1_DDR |= (1<<W1_PIN);
  } else
	W1_DDR &= ~(1<<W1_PIN);
  sei();
  return b;
}


uint w1_byte_wr( uchar b )
{
  uchar i = 8, j;
  bit cnv = (b == CONVERT_T);
  do {
    j = w1_bit_io( b & 1, cnv && i == 1 );
    b >>= 1;
    if( j )
      b |= 0x80;
  } while( --i );
  return b;
}


uint w1_byte_rd( void )
{
  return w1_byte_wr( 0xFF );
}


uchar w1_rom_search( uchar diff, uchar idata *id )
{
  uchar i, j, next_diff;
  bit b;

  if( w1_reset() )
    return PRESENCE_ERR;			// error, no device found
  w1_byte_wr( SEARCH_ROM );			// ROM search command
  next_diff = LAST_DEVICE;			// unchanged on last device
  i = 8 * 8;						// 8 bytes
  do {
    j = 8;							// 8 bits
    do {
      b = w1_bit_io( 1, 0 );		// read bit
	  wait_x_1u();		// 1 us
	  wait_x_1u();		// 1 us
      if( w1_bit_io( 1, 0 ) ) {	// read complement bit
		if( b )						// 11
			return DATA_ERR;		// data error
	  } else {
		if( !b ) {					// 00 = 2 devices
			if( diff > i ||
				((*id & 1) && diff != i) ) {
				b = 1;				// now 1
				next_diff = i;		// next pass 0
			}
		}
      }
      w1_bit_io( b, 0 );     		// write bit
      *id >>= 1;
      if( b )						// store bit
		*id |= 0x80;
      i--;
    } while( --j );
    id++;							// next byte
  } while( i );
  return next_diff;					// to continue search
}


void w1_command( uchar command, uchar idata *id )
{
  uchar i;
  w1_reset();
  if( id ) {
    w1_byte_wr( MATCH_ROM );			// to a single device
    i = 8;
    do {
      w1_byte_wr( *id );
      id++;
    } while( --i );
  } else {
    w1_byte_wr( SKIP_ROM );			// to all devices
  }
  w1_byte_wr( command );
}
